
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <linux/sockios.h>
#include <linux/if_ether.h>
#include <linux/if_arp.h>


typedef struct padi_packet 
{
	u_char Dst_e_hw[ETH_ALEN];
	u_char Src_e_hw[ETH_ALEN];
	u_short frame_type;
	
	u_char v_t;
	u_char code;
	u_short session_id;
	u_short len;
	u_short tag_type;
	u_short tag_len;
} __attribute__ ((packed)) PADI_PACKET;


typedef struct arp_packet
{
	u_char Dst_e_hw[ETH_ALEN];
	u_char Src_e_hw[ETH_ALEN];
	u_short frame_type;
	u_short hw_type;
	u_short prot_type;
	u_char hw_addr_size;
	u_char prot_addr_size;
	u_short op;
	u_char Src_a_hw[ETH_ALEN];
	u_char Src_a_ip[4];
	u_char Dst_a_hw[ETH_ALEN];
	u_char Dst_a_ip[4];
} __attribute__ ((packed)) ARP_PACKET;

void die(char *str);
void get_hw_addr(char *buf, char *str);
void SendArpPkt(char *dev, char *src_e_hw, char *dst_e_hw, int op, char *src_a_hw, char *src_a_ip, char *dst_a_hw, char *dst_a_ip);
void usage();
int getdevindex(int sock, char *name);
int strip(unsigned char *dst, char *str);

void usage()
{
	fprintf(stderr, "* Usage : <dev> <SEmac> <DEmac> <Op> <SAmac> <Sip> <DAmac> <Dip> *\n"
					"* <dev> : Should we send the packet over something *\n"
					"* <SEmac> : source mac address for ethernet header *\n"
					"* <DEmac> : dest mac address for ethernet header *\n"
					"* <Op> : Opcode for arp header read my or cat if_arp.h *\n"
					"* <SAmac> : source mac address for arp header *\n"
					"* <Sip> : source ip address for arp header *\n"
					"* <DAmac> : dest mac address for arp header *\n"
					"* <Dip> : dest ip address for arp header *\n");
	exit(0);
}

main (int argc, char *argv[])
{
	//if (argc<8) usage();
	if (argc < 3) usage();
	fprintf(stderr, "\nLet's go boy... :p\n\n");
	//SendArpPkt(argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7], argv[8]);
    SendpadiPkt(argv[1], argv[2], argv[3]);
}

void die(char *str)
{
	fprintf(stderr, "\nAttention exit for the followent reason : %s\n", str);
	exit(0);
}

int getdevindex(int sock, char *name)
{
	struct ifreq ifr;

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, name, sizeof(ifr.ifr_name));

	if(ioctl(sock, SIOCGIFINDEX, &ifr) == -1) 
		die("I can't get device flags...");
	fprintf(stderr, "Get ethernet flags...\n");
	return ifr.ifr_ifindex;
}

void get_hw_addr(char *buf, char *str)
{
	int i;
	char c, val;

	for (i=0; i < ETH_ALEN; i++) {
		if (!(c=tolower(*str++))) die("Invalid hardware address");
		if (isdigit(c)) val=c-'0';
		else if (c >= 'a' && c <= 'f') val = c-'a'+10;
		else die("Invalid hardware address");
		*buf = val << 4;
		if (!(c=tolower(*str++))) die("Invalid hardware address");
		if (isdigit(c)) val=c-'0';
		else if (c >= 'a' && c <= 'f') val = c-'a'+10;
		else die("Invalid hardware address");
		*buf++ |= val;
		if (*str == ':') str++;
	}
}

int strip(unsigned char *dst, char *str)
{
	if (sscanf(str, "%d.%d.%d.%d", &dst[0], &dst[1], &dst[2], &dst[3]) == 4)
		return 0;
	return -1; 
			
}

void SendArpPkt(char *dev, char *src_e_hw, char *dst_e_hw, int op, char *src_a_hw, char *src_a_ip, char *dst_a_hw, char *dst_a_ip)
{
	struct sockaddr_ll to;
	ARP_PACKET pkt;
	int sock;
	struct ifreq ifr;
	int i;


	if ((sock = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0)
		die("Can't create socket...");
	memset(&ifr, 0, sizeof(ifr));
	ifr.ifr_addr.sa_family = AF_INET;
	strcpy(ifr.ifr_name, "eth0");
	if (ioctl(sock, SIOCGIFINDEX, &ifr) == 0) 
		printf("ifr.ifr_ifindex = %d\n", ifr.ifr_ifindex);
		
		
	ifr.ifr_ifindex = 2;
	if (ioctl(sock, SIOCGIFNAME, &ifr) == 0)
		printf("ifr.ifr_name = %s\n", ifr.ifr_name);
	
	close(sock);
	
	if ((sock = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL))) == -1)
		die("Can't create socket");

	//printf("dev = %s\n", dev);
	if ((to.sll_ifindex = getdevindex(sock, dev)) == -1) {
		close(sock);
		die("Can't get interface");
	}
	
	/* Eth header */
	get_hw_addr(pkt.Src_e_hw, src_e_hw);
	get_hw_addr(pkt.Dst_e_hw, dst_e_hw);
	pkt.frame_type = htons(ETH_P_ARP);
	fprintf(stderr, "Eth header is ready...\n");

	/* arp */
#define ARPHDR_ETHER		0x01
	pkt.hw_type = htons(ARPHDR_ETHER);
	pkt.prot_type = htons(ETH_P_IP);
	pkt.hw_addr_size = ETH_ALEN;
	pkt.prot_addr_size = 4;
	//pkt.op = htons(op);
	pkt.op = htons(0x01);
	fprintf(stderr, "Arp header is ready...\n");

	/* data */
	//get_hw_addr(pkt.Src_a_hw, src_a_hw);
	//get_hw_addr(pkt.Dst_a_hw, dst_a_hw);
	strip(pkt.Src_a_ip, src_a_ip);
	strip(pkt.Dst_a_ip, dst_a_ip);
	get_hw_addr(pkt.Src_a_hw, src_a_hw);
	get_hw_addr(pkt.Dst_a_hw, dst_a_hw);
	
	/* write and send */
	/*for (i = 0; i < sizeof(pkt); i++) {
		unsigned char *ptr;

		ptr = (unsigned char *)&pkt;
		printf("%02x ", ptr[i]);
	}
	printf("\n");*/
	
	if (sendto(sock, &pkt, sizeof(pkt), 0, (struct sockaddr *)&to, sizeof(to)) < 0) {
		close(sock);
		die("I can't send pkt");
	}
	fprintf(stderr, "The packet is send. The matrix is around you!\n");
	close(sock);
	
}
	

void SendpadiPkt(u_char *dev, char *src_e_hw, char *dst_e_hw)
{

	struct sockaddr_ll to;
	struct sockaddr_ll sock_fd;
	PADI_PACKET pkt;
	int sock;
	struct ifreq ifr;
	int i;
	unsigned char packet[100];
	int bytes;

	//if ((sock = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL))) == -1)
	if ((sock = socket(PF_PACKET, SOCK_RAW, htons(0x8863))) == -1)
		die("Can't create socket");

	printf("dev = %s\n", dev);
	if ((to.sll_ifindex = getdevindex(sock, dev)) == -1) {
		close(sock);
		die("Can't get interface");
	}

	/* bind */
	sock_fd.sll_family = AF_PACKET;
	sock_fd.sll_protocol = htons(0x0);
	sock_fd.sll_ifindex = to.sll_ifindex;
	if (bind(sock, (struct sockaddr *)&sock_fd, sizeof(sock_fd)) < 0)
		die("bind call failed");
	
	/* Eth header */
	get_hw_addr(pkt.Src_e_hw, src_e_hw);
	get_hw_addr(pkt.Dst_e_hw, dst_e_hw);
	pkt.frame_type = htons(0x8863);
	fprintf(stderr, "Eth header is ready...\n");
	
	pkt.v_t = 0x11;
	pkt.code = 0x09;
	pkt.session_id = htons(0x0);
	pkt.len = htons(0x04);
	pkt.tag_type = htons(0x0101);
	pkt.tag_len = htons(0x0);

	/* write and send */
	/*for (i = 0; i < sizeof(pkt); i++) {
		unsigned char *ptr;

		ptr = (unsigned char *)&pkt;
		printf("%02x ", ptr[i]);
	}
	printf("\n");*/
	
	if (sendto(sock, &pkt, sizeof(pkt), 0, (struct sockaddr *)&to, sizeof(to)) < 0) {
		close(sock);
		die("I can't send pkt");
	}
	fprintf(stderr, "The packet is send. The matrix is around you!\n");

#if 0
	/* recv */
	bytes = read(sock, &packet, sizeof(packet));
	if (bytes > 0) {
		printf("recv data : ");
		for (i = 0; i < bytes; i++) {
			printf("%02x ", packet[i]);
		}
		printf("\n");
	}
#endif	
					
	
	close(sock);

}
